using System;
using System.IO;
using System.Collections;
//using org.bouncycastle.io;

namespace org.bouncycastle.asn1
{
	public class DERBoolean: DERObject
	{
		byte         value;

		public static readonly DERBoolean FALSE = new DERBoolean(false);
		public static readonly DERBoolean TRUE  = new DERBoolean(true);

		/**
		* return a bool from the passed in object.
		*
		* @exception ArgumentException if the object cannot be converted.
		*/
		public static DERBoolean getInstance(
			Object  obj)
		{
			if ((obj == null) || (typeof(DERBoolean).IsInstanceOfType(obj)))
			{
				return (DERBoolean)obj;
			}

			if (typeof(ASN1OctetString).IsInstanceOfType(obj))
			{
				return new DERBoolean(((ASN1OctetString)obj).getOctets());
			}

			if (typeof(ASN1TaggedObject).IsInstanceOfType(obj))
			{
				return getInstance(((ASN1TaggedObject)obj).getObject());
			}

			throw new ArgumentException("illegal object in getInstance: " + obj.GetType().ToString());
		}

		/**
		* return a DERBoolean from the passed in bool.
		*/
		public static DERBoolean getInstance(
			bool  value)
		{
			return (value ? TRUE : FALSE);
		}

		/**
		* return a Boolean from a tagged object.
		*
		* @param obj the tagged object holding the object we want
		* @param flagExplicit true if the object is meant to be explicitly
		*              tagged false otherwise.
		* @exception ArgumentException if the tagged object cannot
		*               be converted.
		*/
		public static DERBoolean getInstance(
			ASN1TaggedObject obj,
			bool          flagExplicit)
		{
			return getInstance(obj.getObject());
		}
	    
		public DERBoolean(
			byte[]       value)
		{
			this.value = value[0];
		}

		public DERBoolean(
			bool     value)
		{
			this.value = (value) ? (byte)0xff : (byte)0;
		}

		public bool isTrue()
		{
			return (value != 0);
		}

		public override void encode(DEROutputStream outStream)
			//throws IOException
		{
			byte[]  bytes = new byte[1];

			bytes[0] = value;

			outStream.writeEncoded(DERTags.BOOLEAN, bytes);
		}
	    
		public override  bool Equals(Object  o)
		{
			if ((o == null) || !(typeof(DERBoolean).IsInstanceOfType(o)))
			{
				return false;
			}

			return (value == ((DERBoolean)o).value);

		}

		public override int GetHashCode()
		{
			return base.GetHashCode();
		}

	}


}