using System;
using System.IO;
using System.Collections;
using org.bouncycastle.io;

namespace org.bouncycastle.asn1
{
	public class DERConstructedSet: ASN1Set
	{
		public DERConstructedSet()
		{
		}

		/**
		* @param obj - a single object that makes up the set.
		*/
		public DERConstructedSet(
			DEREncodable   obj)
		{
			this.addObject(obj);
		}

		/**
		* @param v - a vector of objects making up the set.
		*/
		public DERConstructedSet(
			DEREncodableVector   v)
		{
			for (int i = 0; i != v.size(); i++)
			{
				this.addObject(v.get(i));
			}
		}

		public override void addObject(
			DEREncodable    obj)
		{
			base.addObject(obj);
		}

		public int getSize()
		{
			return size();
		}

		/*
		* A note on the implementation:
		* <p>
		* As DER requires the constructed, definite-length model to
		* be used for structured types, this varies slightly from the
		* ASN.1 descriptions given. Rather than just outputing SET,
		* we also have to specify CONSTRUCTED, and the objects length.
		*/
		public override void encode(DEROutputStream outStream)
			//throws IOException
		{
			MemoryStream   bOut = new MemoryStream();
			DEROutputStream         dOut = new DEROutputStream(bOut);
			IEnumerator             e = this.getObjects();

			while (e.MoveNext())
			{
				Object    obj = e.Current;

				dOut.writeObject(obj);
			}

			dOut.close();

			byte[]  bytes = bOut.ToArray();

			outStream.writeEncoded(DERTags.SET | DERTags.CONSTRUCTED, bytes);
		}
	}

}