using System;
using System.IO;
using System.Collections;
//using org.bouncycastle.io;

namespace org.bouncycastle.asn1
{
	/**
	* DER NumericString object - this is an ascii string of characters {0,1,2,3,4,5,6,7,8,9, }.
	*/
	public class DERNumericString: DERObject, DERString
	{
		String  m_string;

		/**
		* return a Numeric string from the passed in object
		*
		* @exception ArgumentException if the object cannot be converted.
		*/
		public static DERNumericString getInstance(
			Object  obj)
		{
			if ((obj == null) || (typeof(DERNumericString).IsInstanceOfType(obj)))
			{
				return (DERNumericString)obj;
			}

			if (typeof(ASN1OctetString).IsInstanceOfType(obj))
			{
				return new DERNumericString(((ASN1OctetString)obj).getOctets());
			}

			if (typeof(ASN1TaggedObject).IsInstanceOfType(obj))
			{
				return getInstance(((ASN1TaggedObject)obj).getObject());
			}

			throw new ArgumentException("illegal object in getInstance: " + obj.GetType().ToString());
		}

		/**
		* return an Numeric String from a tagged object.
		*
		* @param obj the tagged object holding the object we want
		* @param flagExplicit true if the object is meant to be explicitly
		*              tagged false otherwise.
		* @exception ArgumentException if the tagged object cannot
		*               be converted.
		*/
		public static DERNumericString getInstance(
			ASN1TaggedObject obj,
			bool          flagExplicit)
		{
			return getInstance(obj.getObject());
		}

		/**
		* basic constructor - with bytes.
		*/
		public DERNumericString(
			byte[]   in_string)
		{
			char[]  cs = new char[in_string.Length];

			for (int i = 0; i != cs.Length; i++)
			{
				cs[i] = (char)(in_string[i] & 0xff);
			}

			this.m_string = new String(cs);
		}

		/**
		* basic constructor - with string.
		*/
		public DERNumericString(
			String   in_string)
		{
			this.m_string = in_string;
		}

		public String getString()
		{
			return m_string;
		}

		public byte[] getOctets()
		{
			char[]  cs = m_string.ToCharArray();
			byte[]  bs = new byte[cs.Length];

			for (int i = 0; i != cs.Length; i++)
			{
				bs[i] = (byte)cs[i];
			}

			return bs; 
		}


		public override void encode(DEROutputStream  outStream)
			//throws IOException
		{
			outStream.writeEncoded(DERTags.NUMERIC_STRING, this.getOctets());
		}


		public override int GetHashCode()
		{
			return this.getString().GetHashCode();
		}

		public override  bool Equals(Object  o)
		{
			if (!(typeof(DERNumericString).IsInstanceOfType(o)))
			{
				return false;
			}

			DERNumericString  s = (DERNumericString)o;

			return this.getString().Equals(s.getString());
		}
	}

}