using System;
using System.IO;
using org.bouncycastle.io;

namespace org.bouncycastle.asn1
{
	public class DEROutputStream: FilterOutputStream 
	{
		public DEROutputStream(
			Stream    os) : base(os)
		{ }

		private void writeLength(
			int length)
			//throws IOException
		{
			if (length > 127)
			{
				int size = 1;
				int val = length;

				while ((val = (int) ((uint) val >> 8)) != 0)
				{
					size++;
				}

				write((byte)(size | 0x80));

				for (int i = (size - 1) * 8; i >= 0; i -= 8)
				{
					write((byte)(length >> i));
				}
			}
			else
			{
				write((byte)length);
			}
		}

		internal void writeEncoded(
			int     tag,
			byte[]  bytes)
			//throws IOException
		{
			write(tag);
			writeLength(bytes.Length);
			write(bytes);
		}

		protected void writeNull()
			//throws IOException
		{
			write(DERTags.NULL);
			write(0x00);
		}

		public virtual void writeObject(
			Object    obj)
			//throws IOException
		{
			if (obj == null)
			{
				writeNull();
			}
			else if (typeof(DERObject).IsInstanceOfType(obj))
			{
				((DERObject)obj).encode(this);
			}
			else if (typeof(DEREncodable).IsInstanceOfType(obj))
			{
				((DEREncodable)obj).getDERObject().encode(this);
			}
			else 
			{
				throw new IOException("object not DEREncodable");
			}
		}
	}

}