using System;
using System.IO;
using System.Collections;
using org.bouncycastle.io;

namespace org.bouncycastle.asn1
{
	/**
	* A DER encoded set object
	*/
	public class DERSet: ASN1Set
	{
		/**
		* create an empty set
		*/
		public DERSet()
		{
		}

		/**
		* @param obj - a single object that makes up the set.
		*/
		public DERSet(
			DEREncodable   obj)
		{
			this.addObject(obj);
		}

		/**
		* @param v - a vector of objects making up the set.
		*/
		public DERSet(
			DEREncodableVector   v)
		{
			for (int i = 0; i != v.size(); i++)
			{
				this.addObject(v.get(i));
			}
		}

		/*
		* A note on the implementation:
		* <p>
		* As DER requires the constructed, definite-length model to
		* be used for structured types, this varies slightly from the
		* ASN.1 descriptions given. Rather than just outputing SET,
		* we also have to specify CONSTRUCTED, and the objects length.
		*/
		public override void encode(DEROutputStream outStream)
		//throws IOException
		{
			MemoryStream   bOut = new MemoryStream();
			DEROutputStream         dOut = new DEROutputStream(bOut);
			IEnumerator             e = this.getObjects();

			while (e.MoveNext())
			{
				Object    obj = e.Current;

				dOut.writeObject(obj);
			}

			dOut.close();

			byte[]  bytes = bOut.ToArray();

			outStream.writeEncoded(DERTags.SET | DERTags.CONSTRUCTED, bytes);
		}
	}

}