using System;
using System.IO;
using System.Text;
//using org.bouncycastle.io;

namespace org.bouncycastle.asn1
{
	/**
	* DER T61String (also the teletex string)
	*/
	public class DERT61String: DERObject, DERString
	{
		String  m_string;

		/**
		* return a T61 string from the passed in object.
		*
		* @exception ArgumentException if the object cannot be converted.
		*/
		public static DERT61String getInstance(
			Object  obj)
		{
			if ((obj == null) || (typeof(DERT61String).IsInstanceOfType(obj)))
			{
				return (DERT61String)obj;
			}

			if (typeof(ASN1OctetString).IsInstanceOfType(obj))
			{
				return new DERT61String(((ASN1OctetString)obj).getOctets());
			}

			if (typeof(ASN1TaggedObject).IsInstanceOfType(obj))
			{
				return getInstance(((ASN1TaggedObject)obj).getObject());
			}

			throw new ArgumentException("illegal object in getInstance: " + obj.GetType().ToString());
		}

		/**
		* return an T61 String from a tagged object.
		*
		* @param obj the tagged object holding the object we want
		* @param explicit true if the object is meant to be explicitly
		*              tagged false otherwise.
		* @exception ArgumentException if the tagged object cannot
		*               be converted.
		*/
		public static DERT61String getInstance(
			ASN1TaggedObject obj,
			bool          flagExplicit)
		{
			return getInstance(obj.getObject());
		}

		/**
		* basic constructor - with bytes.
		*/
		public DERT61String(
			byte[]   in_string)
		{
			StringBuilder s = new StringBuilder();
			for (int i = 0; i < in_string.Length; i++)
			{
				s.Append(in_string[i]);
			}
			this.m_string = s.ToString();
		}

		/**
		* basic constructor - with string.
		*/
		public DERT61String(
			String   in_string)
		{
			this.m_string = in_string;
		}

		public String getString()
		{
			return m_string;
		}

		public override void encode(DEROutputStream  outStream)
			//throws IOException
		{
			char[]  cs = m_string.ToCharArray();
			byte[]  bs = new byte[cs.Length];

			for (int i = 0; i != cs.Length; i++)
			{
				bs[i] = Convert.ToByte(cs[i]);
			}
			outStream.writeEncoded(DERTags.T61_STRING,bs);
		}

		public override bool Equals(Object  o)
		{
			if ((o == null) || !(typeof(DERT61String).IsInstanceOfType(o)))
			{
				return false;
			}

			return this.getString().Equals(((DERT61String)o).getString());
		}

		public override int GetHashCode()
		{
			return base.GetHashCode();
		}

	}

}