using System;
using System.IO;
using System.Collections;
//using org.bouncycastle.io;

namespace org.bouncycastle.asn1
{
	/**
	* DER VisibleString object.
	*/
	public class DERVisibleString: DERObject, DERString
	{
		String  m_string;

		/**
		* return a Visible String from the passed in object.
		*
		* @exception ArgumentException if the object cannot be converted.
		*/
		public static DERVisibleString getInstance(
			Object  obj)
		{
			if ((obj == null) || (typeof(DERVisibleString).IsInstanceOfType(obj)))
			{
				return (DERVisibleString)obj;
			}

			if (typeof(ASN1OctetString).IsInstanceOfType(obj))
			{
				return new DERVisibleString(((ASN1OctetString)obj).getOctets());
			}

			if (typeof(ASN1TaggedObject).IsInstanceOfType(obj))
			{
				return getInstance(((ASN1TaggedObject)obj).getObject());
			}

			throw new ArgumentException("illegal object in getInstance: " + obj.GetType().ToString());
		}

		/**
		* return a Visible String from a tagged object.
		*
		* @param obj the tagged object holding the object we want
		* @param explicit true if the object is meant to be explicitly
		*              tagged false otherwise.
		* @exception ArgumentException if the tagged object cannot
		*               be converted.
		*/
		public static DERVisibleString getInstance(
			ASN1TaggedObject obj,
			bool          flagExplicit)
		{
			return getInstance(obj.getObject());
		}

		/**
		* basic constructor - byte encoded string.
		*/
		public DERVisibleString(
			byte[]   in_string)
		{
			char[]  cs = new char[in_string.Length];

			for (int i = 0; i != cs.Length; i++)
			{
				cs[i] = (char)(in_string[i] & 0xff);
			}

			this.m_string = new String(cs);
		}

		/**
		* basic constructor
		*/
		public DERVisibleString(
			String   in_string)
		{
			this.m_string = in_string;
		}

		public String getString()
		{
			return m_string;
		}

		public byte[] getOctets()
		{
			char[]  cs = m_string.ToCharArray();
			byte[]  bs = new byte[cs.Length];

			for (int i = 0; i != cs.Length; i++)
			{
				bs[i] = (byte)cs[i];
			}

			return bs;
		}

		public override void encode(DEROutputStream  outStream)
			//throws IOException
		{
			outStream.writeEncoded(DERTags.VISIBLE_STRING, this.getOctets());
		}
   
		public override bool Equals(
			Object  o)
		{
			if ((o == null) || !(typeof(DERVisibleString).IsInstanceOfType(o)))
			{
				return false;
			}

			return this.getString().Equals(((DERVisibleString)o).getString());
		}

		public override int GetHashCode()
		{
			return base.GetHashCode();
		}

	}


}