using System;
using System.Collections;
using org.bouncycastle.asn1;

namespace org.bouncycastle.asn1.pkcs
{
	public class ContentInfo: DEREncodable
	{
		private DERObjectIdentifier contentType;
		private DEREncodable        content;

		public static ContentInfo getInstance(
			Object  obj)
		{
			if (typeof(ContentInfo).IsInstanceOfType(obj))
			{
				return (ContentInfo)obj;
			}
			else if (typeof(ASN1Sequence).IsInstanceOfType(obj))
			{
				return new ContentInfo((ASN1Sequence)obj);
			}

			throw new ArgumentException("unknown object in factory");
		}

		public ContentInfo(
			ASN1Sequence  seq)
		{
			IEnumerator   e = seq.getObjects();

			e.MoveNext();
			contentType = (DERObjectIdentifier)e.Current;

			if (e.MoveNext())
			{
				content = ((DERTaggedObject)e.Current).getObject();
			}
		}

		public ContentInfo(
			DERObjectIdentifier contentType,
			DEREncodable        content)
		{
			this.contentType = contentType;
			this.content = content;
		}

		public DERObjectIdentifier getContentType()
		{
			return contentType;
		}

		public DEREncodable getContent()
		{
			return content;
		}

		/**
		 * <pre>
		 * ContentInfo ::= SEQUENCE {
		 *          contentType ContentType,
		 *          content
		 *          [0] EXPLICIT ANY DEFINED BY contentType OPTIONAL }
		 * </pre>
		 */
		public DERObject getDERObject()
		{
			ASN1EncodableVector  v = new ASN1EncodableVector();

			v.add(contentType);

			if (content != null)
			{
				v.add(new BERTaggedObject(0, content));
			}

			return new BERSequence(v);
		}
	}

}