using System;
using System.Collections;
using org.bouncycastle.math;
using org.bouncycastle.asn1;

namespace org.bouncycastle.asn1.pkcs
{
	public class PBKDF2Params: KeyDerivationFunc
	{
		DERObjectIdentifier id;
		ASN1OctetString     octStr;
		DERInteger          iterationCount;
		DERInteger          keyLength;

		internal PBKDF2Params(
			ASN1Sequence  seq): base(seq)
		{
			IEnumerator e = seq.getObjects();
			e.MoveNext();
			id = (DERObjectIdentifier)e.Current;
			e.MoveNext();
			ASN1Sequence  parameters = (ASN1Sequence)e.Current;

			e = parameters.getObjects();

			e.MoveNext();
			octStr = (ASN1OctetString)e.Current;
			e.MoveNext();
			iterationCount = (DERInteger)e.Current;

			if (e.MoveNext())
			{
				keyLength = (DERInteger)e.Current;
			}
			else
			{
				keyLength = null;
			}
		}

		public byte[] getSalt()
		{
			return octStr.getOctets();
		}

		public BigInteger getIterationCount()
		{
			return iterationCount.getValue();
		}

		public BigInteger getKeyLength()
		{
			if (keyLength != null)
			{
				return keyLength.getValue();
			}

			return null;
		}

		public override  DERObject getDERObject()
		{
			ASN1EncodableVector  v = new ASN1EncodableVector();
			ASN1EncodableVector  subV = new ASN1EncodableVector();

			v.add(id);
			subV.add(octStr);
			subV.add(iterationCount);

			if (keyLength != null)
			{
				subV.add(keyLength);
			}

			v.add(new DERSequence(subV));

			return new DERSequence(v);
		}
	}

}