
using System;
//using System.Collections;
using org.bouncycastle.math;
using org.bouncycastle.asn1;

namespace org.bouncycastle.asn1.x509
{
	public class BasicConstraints: DEREncodable
	{
		DERBoolean  cA = new DERBoolean(false);
		DERInteger  pathLenConstraint = null;

		public static BasicConstraints getInstance(
			ASN1TaggedObject obj,
			bool          flagExplicit)
		{
			return getInstance(ASN1Sequence.getInstance(obj, flagExplicit));
		}

		public static BasicConstraints getInstance(
			Object  obj)
		{
			
			if ((obj == null) || (typeof(BasicConstraints).IsInstanceOfType(obj)))
			{
				return (BasicConstraints)obj;
			}
			else if (typeof(ASN1Sequence).IsInstanceOfType(obj))
			{
				return new BasicConstraints((ASN1Sequence)obj);
			}

			throw new ArgumentException("unknown object in factory");
		}
		
		public BasicConstraints(
			ASN1Sequence   seq)
		{
			this.cA = (DERBoolean)seq.getObjectAt(0);
			if (seq.size() > 1)
			{
				this.pathLenConstraint = (DERInteger)seq.getObjectAt(1);
			}
		}

		public BasicConstraints(
			bool cA,
			int     pathLenConstraint)
		{
			this.cA = new DERBoolean(cA);
			this.pathLenConstraint = new DERInteger(pathLenConstraint);
		}

		public BasicConstraints(
			bool cA)
		{
			this.cA = new DERBoolean(cA);
			this.pathLenConstraint = null;
		}

		public bool isCA()
		{
			return cA.isTrue();
		}

		public BigInteger getPathLenConstraint()
		{
			if (pathLenConstraint != null)
			{
				return pathLenConstraint.getValue();
			}

			return null;
		}

		/**
		* <pre>
		* BasicConstraints := SEQUENCE {
		*    cA                  BOOLEAN DEFAULT FALSE,
		*    pathLenConstraint   INTEGER (0..MAX) OPTIONAL
		* }
		* </pre>
		*/
		public DERObject getDERObject()
		{
			ASN1EncodableVector  v = new ASN1EncodableVector();

			v.add(cA);

			if (pathLenConstraint != null)
			{
				v.add(pathLenConstraint);
			}

			return new DERSequence(v);
		}

		public override String ToString()
		{
			return "BasicConstraints: isCa(" + this.isCA() + "), pathLenConstraint = " + pathLenConstraint.getValue();
		}
	}

}