
using System;
//using System.Collections;
//using org.bouncycastle.io;
using org.bouncycastle.asn1;

namespace org.bouncycastle.asn1.x509
{
	public class ReasonFlags: DERBitString
	{
		public static readonly int UNUSED                  = (1 << 7);
		public static readonly int KEY_COMPROMISE          = (1 << 6);
		public static readonly int CA_COMPROMISE           = (1 << 5);
		public static readonly int AFFILIATION_CHANGED     = (1 << 4);
		public static readonly int SUPERSEDED              = (1 << 3);
		public static readonly int CESSATION_OF_OPERATION  = (1 << 2);
		public static readonly int CERTIFICATE_HOLD        = (1 << 1);
		public static readonly int PRIVILEGE_WITHDRAWN     = (1 << 0);
		public static readonly int AA_COMPROMISE           = (1 << 15);

		/**
		 * <pre>
		 * ReasonFlags ::= BIT STRING {
		 *    unused(0),
		 *    keyCompromise(1),
		 *    cACompromise(2),
		 *    affiliationChanged(3),
		 *    superseded(4),
		 *    cessationOfOperation(5),
		 *    certficateHold(6)
		 * }
		 * </pre>
		 * @param reasons - the bitwise OR of the Key Reason flags giving the
		 * allowed uses for the key.
		 */
		public ReasonFlags(
			int reasons): base(getBytes(reasons), getPadBits(reasons)) {}

		public ReasonFlags(
			DERBitString reasons): base(reasons.getBytes(), reasons.getPadBits()) {}
	}

}